-- Copyright (c) 2011, Pedro Ignacio Martos <pmartos@fi.uba.ar / pimartos@gmail.com> & Fabricio Baglivo <baglivofabricio@gmail.com>
-- All rights reserved.
--  
-- Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-- the following conditions are met:
-- 
--     * Redistributions of source code must retain the above copyright notice, this list of conditions and the 
--       following disclaimer.
--     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and 
--       the following disclaimer in the documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
-- DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
-- SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
-- USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity CM0_DSSystem is
    Port ( PMOD1_0_LS   : out  STD_LOGIC; -- dcm
           PMOD1_1_LS   : out  STD_LOGIC; -- sleep
           PMOD1_2_LS   : out  STD_LOGIC; -- lock
           PMOD1_3_LS   : out  STD_LOGIC; -- detector
           PMOD2_0_LS   : out  STD_LOGIC; -- reset
           PMOD2_1_LS   : out  STD_LOGIC; -- inst_mem_enable
           PMOD2_2_LS   : out  STD_LOGIC;
           PMOD2_3_LS   : out  STD_LOGIC;
           SYSCLK_P     : in  STD_LOGIC;
           SYSCLK_N     : in  STD_LOGIC;
           GPIO_DIP_SW0 : in STD_LOGIC; -- GLOBAL ENABLE
           GPIO_SW_N    : in STD_LOGIC -- reset push button
                      );
end CM0_DSSystem;

architecture Behavioral of CM0_DSSystem is
attribute mark_debug : string;
component Memory
	port (
	clka: IN std_logic;
	rsta: IN std_logic;
	ena: IN std_logic;
	wea: IN std_logic_VECTOR(0 downto 0);
	addra: IN std_logic_VECTOR(12 downto 0);
	dina: IN std_logic_VECTOR(31 downto 0);
	douta: OUT std_logic_VECTOR(31 downto 0));
end component;

component DetectorBus is
    Port ( Clock : in  STD_LOGIC;
           SynqReset:in STD_LOGIC;
           DataBus : in  STD_LOGIC_VECTOR (31 downto 0);
           Detector : out  STD_LOGIC);
end component;

COMPONENT SyncReset 
	PORT(
		clock : IN std_logic;  
		ExtReset : in  std_logic;        
		ResetPulse : OUT std_logic
		);
	END COMPONENT;
	
COMPONENT Loop_Cache_Controller
PORT (
clk : IN std_logic;
n_reset : IN std_logic;
global_cache_enable: IN std_logic;
htrans  : IN std_logic;
hprot : IN std_logic;
new_address : IN std_logic_vector (12 downto 0);
main_cache : OUT std_logic;
cache_write_enable : OUT std_logic
--mux_memory_select : OUT std_logic;
--inst_mem_enablec : OUT std_logic
); 
END COMPONENT;

COMPONENT loop_cache
PORT (
clk : IN std_logic;
n_reset : IN std_logic;
htrans: IN std_logic;
enable_cache : IN std_logic;
write_enable : IN std_logic;
data_in : IN std_logic_vector (31 downto 0);
address : IN std_logic_vector (1 downto 0);
data_out : OUT std_logic_vector (31 downto 0)
);
END COMPONENT;

--COMPONENT Multiplexer
--PORT (
--cache_mem_out : IN std_logic_vector (31 downto 0);
--mux_mem_select : IN std_logic;
--hrdata : OUT std_logic_vector (31 downto 0)
--);
--END COMPONENT

COMPONENT SystemClock
	PORT(
		clk_in1 : IN std_logic;
		clk_out1 : OUT std_logic;
		locked : OUT std_logic
		);
END COMPONENT;
	
COMPONENT CORTEXM0DS 
	PORT(
  -- CLOCK AND RESETS ------------------
  --input  wire        HCLK,              -- Clock
  --input  wire        HRESETn,           -- Asynchronous reset
  HCLK : IN std_logic;              -- Clock
  HRESETn : IN std_logic;           -- Asynchronous reset

  -- AHB-LITE MASTER PORT --------------
  --output wire [31:0] HADDR,             -- AHB transaction address
  --output wire [ 2:0] HBURST,            -- AHB burst: tied to single
  --output wire        HMASTLOCK,         -- AHB locked transfer (always zero)
  --output wire [ 3:0] HPROT,             -- AHB protection: priv; data or inst
  --output wire [ 2:0] HSIZE,             -- AHB size: byte, half-word or word
  --output wire [ 1:0] HTRANS,            -- AHB transfer: non-sequential only
  --output wire [31:0] HWDATA,            -- AHB write-data
  --output wire        HWRITE,            -- AHB write control
  --input  wire [31:0] HRDATA,            -- AHB read-data
  --input  wire        HREADY,            -- AHB stall signal
  --input  wire        HRESP,             -- AHB error response
  HADDR : OUT std_logic_vector (31 downto 0);             -- AHB transaction address
  HBURST : OUT std_logic_vector (2 downto 0);            -- AHB burst: tied to single
  HMASTLOCK : OUT std_logic;         -- AHB locked transfer (always zero)
  HPROT : OUT std_logic_vector (3 downto 0);              -- AHB protection: priv; data or inst
  HSIZE : OUT std_logic_vector (2 downto 0);             -- AHB size: byte, half-word or word
  HTRANS : OUT std_logic_vector (1 downto 0);            -- AHB transfer: non-sequential only
  HWDATA : OUT std_logic_vector (31 downto 0);             -- AHB write-data
  HWRITE : OUT std_logic;            -- AHB write control
  HRDATA : IN std_logic_vector (31 downto 0);            -- AHB read-data
  HREADY : IN std_logic;            -- AHB stall signal
  HRESP : IN std_logic;             -- AHB error response

  -- MISCELLANEOUS ---------------------
  --input  wire        NMI,               -- Non-maskable interrupt input
  --input  wire [15:0] IRQ,               -- Interrupt request inputs
  --output wire        TXEV,              -- Event output (SEV executed)
  --input  wire        RXEV,              -- Event input
  --output wire        LOCKUP,            -- Core is locked-up
  --output wire        SYSRESETREQ,       -- System reset request
  NMI : IN std_logic;               -- Non-maskable interrupt input
  IRQ : IN std_logic_vector (15 downto 0);               -- Interrupt request inputs
  TXEV : OUT std_logic;              -- Event output (SEV executed)
  RXEV : IN std_logic;              -- Event input
  LOCKUP : OUT std_logic;            -- Core is locked-up
  SYSRESETREQ : OUT std_logic;       -- System reset request

  -- POWER MANAGEMENT ------------------
  --output wire        SLEEPING           -- Core and NVIC sleeping
  SLEEPING : OUT std_logic          -- Core and NVIC sleeping
);
END COMPONENT;

signal SyncResetPulse : STD_LOGIC;
signal dummy : STD_LOGIC_VECTOR (2 downto 0);
signal HRData : std_logic_vector (31 downto 0);
signal HWData : std_logic_vector (31 downto 0);
signal HADDR : std_logic_vector (31 downto 0);
signal HBurst : std_logic_vector (2 downto 0);
signal HProt : std_logic_vector (3 downto 0);
signal HSize : std_logic_vector (2 downto 0);
signal HTrans : std_logic_vector (1 downto 0);
signal HWrite : std_logic_vector (0 downto 0);
signal Clock : std_logic;
signal none : std_logic_vector (1 downto 0);
signal led_value: std_logic;
signal reset_rom_detector: std_logic;
signal Clock_in: std_logic;
signal cache_enable: std_logic;
signal cache_write_enable: std_logic;
signal mux_memory_select: std_logic;
signal inst_mem_out: std_logic_vector(31 downto 0);
signal cache_out: std_logic_vector (31 downto 0);
signal global_cache_enable: std_logic;
signal main_cache: std_logic;
signal inst_mem_enable_final: std_logic;
signal global_enable: std_logic;
signal enable_cache: std_logic;
attribute mark_debug of SyncResetPulse: signal is "true";
attribute mark_debug of HRData: signal is "true";
attribute mark_debug of HADDR: signal is "true";
attribute mark_debug of HTrans: signal is "true";
attribute mark_debug of HProt: signal is "true";
attribute mark_debug of led_value: signal is "true";
attribute mark_debug of cache_enable: signal is "true";
attribute mark_debug of cache_write_enable: signal is "true";
attribute mark_debug of mux_memory_select: signal is "true";
attribute mark_debug of inst_mem_out: signal is "true";
attribute mark_debug of cache_out: signal is "true";
attribute mark_debug of global_enable: signal is "true";
attribute mark_debug of inst_mem_enable_final: signal is "true";

begin
PMOD1_3_LS <= led_value;
PMOD2_0_LS <= SyncResetPulse;
--PMOD2_1_LS <= '1';
PMOD2_2_LS <= '0';
PMOD2_3_LS <= '1';
reset_rom_detector <= not SyncResetPulse;
PMOD2_1_LS <= global_enable;
enable_cache <= not (main_cache);

Inst_Detector: DetectorBus 
    Port map ( Clock => Clock,
               SynqReset => reset_rom_detector,
               DataBus => HRData,
               Detector => led_value);
           
Inst_Loop_Cache_Controller: Loop_Cache_Controller
PORT MAP(
clk => Clock,
n_reset => SyncResetPulse,
global_cache_enable => global_enable,
htrans => HTrans (1),
hprot => HProt (0),
new_address => HADDR (14 downto 2),
main_cache => main_cache,
cache_write_enable => cache_write_enable
--mux_memory_select => mux_memory_select,
--inst_mem_enablec => inst_mem_enable
);

Inst_SyncReset: SyncReset PORT MAP(
		clock => Clock,
		ExtReset => GPIO_SW_N,
		ResetPulse => SyncResetPulse
	);

Inst_Loop_Cache: loop_cache PORT MAP(
clk => Clock,
n_reset => SyncResetPulse,
htrans => HTrans(1),
enable_cache => enable_cache,
write_enable => cache_write_enable, 
data_in => inst_mem_out,
address => HADDR (3 downto 2),
data_out => cache_out
);

--process (cache_enable, HTrans(1), GPIO_DIP_SW0, inst_mem_enable)
process (HTrans(1), main_cache, global_enable)
begin
inst_mem_enable_final <= (not global_enable or (global_enable and main_cache ))and HTrans(1);
end process;

process (SyncResetPulse, GPIO_DIP_SW0)
begin
    if rising_edge (SyncResetPulse) then
        global_enable <= GPIO_DIP_SW0;
    end if;
end process;



process (main_cache, inst_mem_out, cache_out)
begin
if (main_cache = '1') then
HRData <= inst_mem_out;
else
HRData <= cache_out;
end if;
end process;

Inst_SystemClock: SystemClock PORT MAP(
		clk_in1 => Clock_in,
		clk_out1 => Clock,
		locked => PMOD1_0_LS
	);
	
Inst_IBUFGDS : IBUFGDS
generic map (
DIFF_TERM => FALSE, -- Differential Termination
IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
IOSTANDARD => "DEFAULT")
port map (
    O => Clock_in, -- Clock buffer output
    I => SYSCLK_P, -- Diff_p clock buffer input (connect directly to top-level port)
    IB => SYSCLK_N -- Diff_n clock buffer input (connect directly to top-level port)
    );

Inst_Memory : Memory
		port map (
			clka => Clock,
			rsta => reset_rom_detector,
			ena => inst_mem_enable_final,
			wea => HWrite (0 downto 0),
			addra => HADDR(14 downto 2),
			dina => HWData(31 downto 0),
			douta => inst_mem_out
			);
			
Processor : CORTEXM0DS	port map (
	-- CLOCK AND RESETS ------------------
  HCLK => Clock,              -- Clock
  HRESETn => SyncResetPulse,           -- Asynchronous reset

  -- AHB-LITE MASTER PORT --------------
  HADDR => HADDR(31 downto 0),             -- AHB transaction address
  HBURST => HBurst(2 downto 0),            -- AHB burst: tied to single
  HMASTLOCK => dummy(0),         -- AHB locked transfer (always zero)
  HPROT => HProt (3 downto 0),             -- AHB protection: priv; data or inst
  HSIZE => HSize(2 downto 0),             -- AHB size: byte, half-word or word
  HTRANS => HTrans (1 downto 0),            -- AHB transfer: non-sequential only
  HWDATA => HWData(31 downto 0),            -- AHB write-data
  HWRITE => HWrite(0),            -- AHB write control
  HRDATA => HRData(31 downto 0),            -- AHB read-data
  HREADY => '1',            -- AHB stall signal
  HRESP => '0',             -- AHB error response

  -- MISCELLANEOUS ---------------------
  NMI => '0',               -- Non-maskable interrupt input
  IRQ => "0000000000000000", --Interrupciones(15 downto 0),               -- Interrupt request inputs
  TXEV => dummy(1),              -- Event output (SEV executed)
  RXEV => '0',              -- Event input
  LOCKUP => PMOD1_2_LS,            -- Core is locked-up
  SYSRESETREQ => dummy(2),       -- System reset request

  -- POWER MANAGEMENT ------------------
  SLEEPING => PMOD1_1_LS           -- Core and NVIC sleeping
	);

end Behavioral; 
