module Loop_Cache_Controller
(
input wire clk,
input wire n_reset,
input wire global_cache_enable,
input wire htrans,
input wire hprot,
input wire [12:0] new_address,
output reg main_cache,
output reg cache_write_enable
//output reg mux_memory_select,
//output reg inst_mem_enablec
);

parameter cache_size=4; // size of the cache in words
localparam IDLE=0, FILL=1, ACTIVE=2;

(* mark_debug = "true" *) reg [1:0] present_state, next_state;
(* mark_debug = "true" *) reg prot_trans;
(* mark_debug = "true" *) reg [12:0] branch;
(* mark_debug = "true" *) reg [12:0] branch_target;
(* mark_debug = "true" *) reg [12:0] previous_address;
(* mark_debug = "true" *) reg loop_detector;


always @(posedge clk or negedge n_reset)
begin
	if (~n_reset)
		previous_address<=13'b0000000000000;
	else
		begin
			if (global_cache_enable)
				previous_address<=new_address;
			else
				previous_address<=13'b0000000000000;
		end
end


always @(posedge clk or negedge n_reset) // ovdje sam promjenila da ne bude clk vec internal_clka
begin
	if (~n_reset)
		present_state <= IDLE;
	else
		present_state <= next_state; 
end

always @(posedge clk or negedge n_reset)
begin
if (~n_reset)
    begin   
    branch<= 13'b0000000000000;
    branch_target<= 13'b0000000000000;
    end
else
    if (loop_detector)
        begin
        branch<=previous_address;
        branch_target<=new_address;
        end
end

always @(*)
begin
next_state=present_state;
main_cache=1'b1;
cache_write_enable=1'b0;
loop_detector=1'b0;
prot_trans=1'b0;
if (global_cache_enable)
	begin
	prot_trans = ~hprot & htrans;
	case (present_state)
		IDLE: begin
			cache_write_enable = 1'b0;
			main_cache = 1'b1;    
			if  (prot_trans && ($signed(previous_address-new_address) <= (cache_size-1)) && ($signed (previous_address-new_address) > 1))
				begin
				next_state = FILL;
				loop_detector=1'b1;
				//branch=previous_address;
				//branch_target=new_address;
				end
			else
				next_state = IDLE;
			end 
      
		FILL: begin 
			  if (~hprot)
				begin
				if ((new_address==branch_target) && (previous_address==branch))
					begin
					next_state = ACTIVE;  ////////////////////////////////////////////////!!!!!!!!!!!!!
					main_cache = 1'b0;  
					cache_write_enable=1'b1;
					end
				else
					begin	  
					if (((new_address==previous_address) || (new_address==previous_address+1)) && (new_address<=branch)) 
						begin
						next_state = FILL;
						cache_write_enable=1'b1;
						main_cache = 1'b1;
						end
					else
						begin
						next_state = IDLE;
						cache_write_enable=1'b1;
						main_cache = 1'b1;
						end
					end  
				end
			  else
				begin
			    cache_write_enable=1'b1;
			    main_cache = 1'b1;
				next_state = IDLE;	        
				end
			  end


		ACTIVE: begin         
				if (~hprot)
					begin 
					if (new_address<=branch && ((new_address==previous_address)|| (new_address==previous_address+1)  || (new_address==branch_target && previous_address==branch)))
						begin
						next_state = ACTIVE;
						cache_write_enable=1'b0;
						main_cache = 1'b0;
						end
					else
						begin
						next_state = IDLE;
						cache_write_enable=1'b0;
						main_cache = 1'b1;
						end
					end
				else
					begin
					next_state = IDLE;
					cache_write_enable=1'b0;
					main_cache = 1'b1; 
					end
				end
				
	endcase
	end
else // default values for the case of global_enable=0
	begin
	cache_write_enable=1'b0;
	main_cache=1'b1;
	next_state = IDLE;
	prot_trans=1'b0;
	loop_detector=1'b0;
	end
end
endmodule